function [f, gg] = feature_norm_glcm(img, segm, pmap, roi, opt, debug)
% feature_glcm - GLCM features
%
%    f = feature_glcm(img, segm, pmap, roi, opt, debug)
%
%    opt = 's' - symmetric (default: not symmetric)
%    opt = 'f' - fixed histogram bins (0-255)
%
%    f(5*(i-1)+1)  = constrast for degree d(i)
%    f(5*(i-1)+2)  = correlation for degree d(i)
%    f(5*(i-1)+3)  = energy for degree d(i)
%    f(5*(i-1)+4)  = homogeneity for degree d(i)
%    f(5*(i-1)+5)  = entropy for degree d(i)
% 
%   where d = [0, 45 ,90, 135] degrees, and i=1:4
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

rs      = double(roi).*double(segm);  % valid region
ii      = find(~rs);                  % find invalid region
tmp     = double(img);
tmp(ii) = nan;                        % mark invalid region

if (any(opt == 'f'))
  myG = [0 255];
else
  myG = [];
end

% compute matrix for four directions [0, 45, 90, 135]
gg = mygraycomatrix(tmp, 8, myG, [0 1; -1 1; -1 0; -1 -1], pmap.pmapxy);

% symmetrize
if (any(opt == 's'))
  for i=1:size(gg,3)
    gg(:,:,i) = gg(:,:,i) + gg(:,:,i)';
  end
end


% compute properties (via MATLAB) -- precise to 0.001
gc = mygraycoprops(round(1000*gg), 'all');

% compute entropy
for i=1:size(gg,3)
  ii = find(gg(:,:,i));                  % find non-zero entries
  if (isempty(ii))
    en(i) = 0;      % zero matrix
  else
    gn = gg(:,:,i) / sum(sum(gg(:,:,i)));  % normalize gg
    en(i) = -sum(gn(ii).*log2(gn(ii)));    % compute entropy
  end
end

f = [];
for i=1:size(gg,3)
  f = [f; ...
       gc.Contrast(i); ...
       gc.Correlation(i); ...
       gc.Energy(i); ...
       gc.Homogeneity(i); ...
       en(i) ];
end
     
if (debug)
  figure
  subplot(2,3,1)
  imagesc(tmp)
  subplot(2,3,4)
  bar(f);
  subplot(2,3,2)
  imagesc(gg(:,:,1))
  subplot(2,3,3)
  imagesc(gg(:,:,2));
  subplot(2,3,5)
  imagesc(gg(:,:,3))
  subplot(2,3,6)
  imagesc(gg(:,:,4));
  
  gc
  en
  f'
end


function GLCMS = mygraycomatrix(I, NL, GL, Offset, pmap)

if isempty(GL)
  GL(1) = min(I(:));
  GL(2) = max(I(:));
end
GL = double(GL);

slope = (NL-1) / (GL(2) - GL(1));
intercept = 1 - (slope*(GL(1)));
SI = round(imlincomb(slope,I,intercept,'double'));
SI(SI > NL) = NL;
SI(SI < 1) = 1;

numOffsets = size(Offset,1);

% Initialize GLCMS array.
GLCMS = zeros(NL,NL,numOffsets);

% Create vectors of row and column subscripts for every pixel and its neighbor.
s = size(I);
[r,c] = meshgrid(1:s(1),1:s(2));
r = r(:);
c = c(:);

% Compute GLCMS
for k = 1 : numOffsets
  GLCMS(:,:,k) = mycomputeGLCM(r,c,Offset(k,:),SI,NL,pmap);
end


function oneGLCM = mycomputeGLCM(r,c,offset,si,nl,pmap)
% computes GLCM given one Offset

  r2 = r + offset(1);
  c2 = c + offset(2);
  
  % Remove pixel and its neighbor if they have subscripts outside the image
  % boundary.
  s = size(si);
  bad = find(c2 < 1 | c2 > s(2) | r2 < 1 | r2 > s(1));
  Index = [r c r2 c2];
  Index(bad,:) = [];
  
  % Create vectors containing the values of r and c (v1 and v2 respectively) for
  % each pixel pair.
  v1 = si(sub2ind(s,Index(:,1),Index(:,2)));
  v2 = si(sub2ind(s,Index(:,3),Index(:,4)));
  
  % Make sure that v1 and v2 are column vectors.
  v1 = v1(:);
  v2 = v2(:);
  
  % Remove pixel and its neighbor if their value is NaN.
  bad = isnan(v1) | isnan(v2);
  %if any(bad)
  %  wid = sprintf('Images:%s:scaledImageContainsNan',mfilename);
  %  msg = 'GLCM does not count pixel pairs if either of their values is NaN.';
  %  warning(wid,'%s', msg);
  %end
  Ind = [v1 v2];
  Ind(bad,:) = [];

  % construct the corresponding pmap
  p1 = pmap(sub2ind(s,Index(:,1),Index(:,2)));
  Pind = p1(:);
  Pind(bad,:) = [];
  
  if isempty(Ind)
    oneGLCM = zeros(nl);
  else
    % Tabulate the occurrences of pixel pairs having v1 and v2.
    oneGLCM = accumarray(Ind, Pind, [nl nl]);
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function stats = mygraycoprops(varargin)
%GRAYCOPROPS Properties of a gray-level co-occurrence matrix.  
%   STATS = GRAYCOPROPS(GLCM,PROPERTIES) normalizes the gray-level
%   co-occurrence matrix (GLCM) so that the sum of its elements is one. Each
%   element in the normalized GLCM, (r,c), is the joint probability occurrence
%   of pixel pairs with a defined spatial relationship having gray level
%   values r and c in the image. GRAYCOPROPS uses the normalized GLCM to
%   calculate PROPERTIES.
%
%   GLCM can be an m x n x p array of valid gray-level co-occurrence
%   matrices. Each gray-level co-occurrence matrix is normalized so that its
%   sum is one.
%
%   PROPERTIES can be a comma-separated list of strings, a cell array
%   containing strings, the string 'all', or a space separated string. They
%   can be abbreviated, and case does not matter.
%
%   Properties include:
%  
%   'Contrast'      the intensity contrast between a pixel and its neighbor 
%                   over the whole image. Range = [0 (size(GLCM,1)-1)^2]. 
%                   Contrast is 0 for a constant image.
%
%   'Correlation'   statistical measure of how correlated a pixel is to its 
%                   neighbor over the whole image. Range = [-1 1]. 
%                   Correlation is 1 or -1 for a perfectly positively or
%                   negatively correlated image. Correlation is NaN for a 
%                   constant image.
%
%   'Energy'        summation of squared elements in the GLCM. Range = [0 1].
%                   Energy is 1 for a constant image.
%  
%   'Homogeneity'   closeness of the distribution of elements in the GLCM to
%                   the GLCM diagonal. Range = [0 1]. Homogeneity is 1 for
%                   a diagonal GLCM.
%  
%   If PROPERTIES is the string 'all', then all of the above properties are
%   calculated. This is the default behavior. Please refer to the
%   Documentation for more information on these properties.
%  
%   STATS is a structure with fields that are specified by PROPERTIES. Each
%   field contains a 1 x p array, where p is the number of gray-level
%   co-occurrence matrices in GLCM. For example, if GLCM is an 8 x 8 x 3 array
%   and PROPERTIES is 'Energy', then STATS is a structure containing the
%   field 'Energy'. This field contains a 1 x 3 array.
%
%   Notes
%   -----  
%   Energy is also known as uniformity, uniformity of energy, and angular second
%   moment.
%
%   Contrast is also known as variance and inertia.
%
%   Class Support
%   -------------  
%   GLCM can be logical or numeric, and it must contain real, non-negative, finite,
%   integers. STATS is a structure.
%
%   Examples
%   --------
%   GLCM = [0 1 2 3;1 1 2 3;1 0 2 0;0 0 0 3];
%   stats = graycoprops(GLCM)
%
%   I = imread('circuit.tif');
%   GLCM2 = graycomatrix(I,'Offset',[2 0;0 2]);
%   stats = graycoprops(GLCM2,{'contrast','homogeneity'})
%  
%   See also GRAYCOMATRIX.

allStats = {'Contrast','Correlation','Energy','Homogeneity'};
  
[glcm, requestedStats] = ParseInputs(allStats, varargin{:});

% Initialize output stats structure.
numStats = length(requestedStats);
numGLCM = size(glcm,3);
empties = repmat({zeros(1,numGLCM)},[numStats 1]);
stats = cell2struct(empties,requestedStats,1);

for p = 1 : numGLCM
  
  if numGLCM ~= 1 %N-D indexing not allowed for sparse. 
    tGLCM = normalizeGLCM(glcm(:,:,p));
  else 
    tGLCM = normalizeGLCM(glcm);
  end
  
  % Get row and column subscripts of GLCM.  These subscripts correspond to the
  % pixel values in the GLCM.
  s = size(tGLCM);
  [c,r] = meshgrid(1:s(1),1:s(2));
  r = r(:);
  c = c(:);

  % Calculate fields of output stats structure.
  for k = 1:numStats
    name = requestedStats{k};  
    switch name
     case 'Contrast'
      stats.(name)(p) = calculateContrast(tGLCM,r,c);
      
     case 'Correlation'
      stats.(name)(p) = calculateCorrelation(tGLCM,r,c);
      
     case 'Energy'
      stats.(name)(p) = calculateEnergy(tGLCM);
      
     case 'Homogeneity'
      stats.(name)(p) = calculateHomogeneity(tGLCM,r,c);
    end
  end

end


%-----------------------------------------------------------------------------
function glcm = normalizeGLCM(glcm)
  
% Normalize glcm so that sum(glcm(:)) is one.
if any(glcm(:))
  glcm = glcm ./ sum(glcm(:));
end
  
%-----------------------------------------------------------------------------
function C = calculateContrast(glcm,r,c)
% Reference: Haralick RM, Shapiro LG. Computer and Robot Vision: Vol. 1,
% Addison-Wesley, 1992, p. 460.  
k = 2;
l = 1;
term1 = abs(r - c).^k;
term2 = glcm.^l;
  
term = term1 .* term2(:);
C = sum(term);

%-----------------------------------------------------------------------------
function Corr = calculateCorrelation(glcm,r,c)
% References: 
% Haralick RM, Shapiro LG. Computer and Robot Vision: Vol. 1, Addison-Wesley,
% 1992, p. 460.
% Bevk M, Kononenko I. A Statistical Approach to Texture Description of Medical
% Images: A Preliminary Study., The Nineteenth International Conference of
% Machine Learning, Sydney, 2002. 
% http://www.cse.unsw.edu.au/~icml2002/workshops/MLCV02/MLCV02-Bevk.pdf, p.3.
  
% Correlation is defined as the covariance(r,c) / S(r)*S(c) where S is the
% standard deviation.

% Calculate the mean and standard deviation of a pixel value in the row
% direction direction. e.g., for glcm = [0 0;1 0] mr is 2 and Sr is 0.
mr = meanIndex(r,glcm);
Sr = stdIndex(r,glcm,mr);
  
% mean and standard deviation of pixel value in the column direction, e.g.,
% for glcm = [0 0;1 0] mc is 1 and Sc is 0.
mc = meanIndex(c,glcm);
Sc = stdIndex(c,glcm,mc);

term1 = (r - mr) .* (c - mc) .* glcm(:);
term2 = sum(term1);

ws = warning('off','Matlab:divideByZero');
Corr = term2 / (Sr * Sc);
warning(ws);

%-----------------------------------------------------------------------------
function S = stdIndex(index,glcm,m)

term1 = (index - m).^2 .* glcm(:);
S = sqrt(sum(term1));

%-----------------------------------------------------------------------------
function M = meanIndex(index,glcm)

M = index .* glcm(:);
M = sum(M);

%-----------------------------------------------------------------------------
function E = calculateEnergy(glcm)
% Reference: Haralick RM, Shapiro LG. Computer and Robot Vision: Vol. 1,
% Addison-Wesley, 1992, p. 460.  
  
foo = glcm.^2;
E = sum(foo(:));

%-----------------------------------------------------------------------------
function H = calculateHomogeneity(glcm,r,c)
% Reference: Haralick RM, Shapiro LG. Computer and Robot Vision: Vol. 1,
% Addison-Wesley, 1992, p. 460.  
  
term1 = (1 + abs(r - c));
term = glcm(:) ./ term1;
H = sum(term);

%-----------------------------------------------------------------------------
function [glcm,reqStats] = ParseInputs(allstats,varargin)
  
numstats = length(allstats);
%iptchecknargin(1,numstats+1,nargin,mfilename);

reqStats = '';
glcm = varargin{1};

% The 'nonnan' and 'finite' attributes are not added to iptcheckinput because the
% 'integer' attribute takes care of these requirements.
%iptcheckinput(glcm,{'logical','numeric'},{'real','nonnegative','integer'}, ...
%              mfilename,'GLCM',1);

if ndims(glcm) > 3
  eid = sprintf('Images:%s:invalidSizeForGLCM',mfilename);
  msg = 'GLCM must either be an m x n or an m x n x p array.';
  error(eid,'%s',msg);
end

% Cast GLCM to double to avoid truncation by data type. Note that GLCM is not an
% image.
if ~isa(glcm,'double')
  glcm = double(glcm);
end

list = varargin(2:end);

if isempty(list)
  % GRAYCOPROPS(GLCM) or GRAYCOPROPS(GLCM,PROPERTIES) where PROPERTIES is empty.
  reqStats = allstats;
else
  if iscell(list{1}) || numel(list) == 1
    % GRAYCOPROPS(GLCM,{...})
    list = list{1};
  end

  if ischar(list)
    %GRAYCOPROPS(GLCM,SPACE-SEPARATED STRING)
    list = strread(list, '%s');
  end

  anyprop = allstats;
  anyprop{end+1} = 'all';
  
  for k = 1 : length(list)
    match = list{k};
    %match = iptcheckstrs(list{k}, anyprop, mfilename, 'PROPERTIES', k+1);
    if strcmp(match,'all')
      reqStats = allstats;
      break;
    end
    reqStats{k} = match;
  end
  
end

% Make sure that reqStats are in alphabetical order.
reqStats = sort(reqStats);

if isempty(reqStats)
  eid = sprintf('Images:%s:internalError',mfilename);
  msg = 'Internal error: requestedStats has not been assigned.';
  error(eid,'%s',msg);
end

