function [out1, out2] = gpr_new(logtheta, covfunc, x, y, xstar);

% gpr - Gaussian process regression, with a named covariance function. Two
% modes are possible: training and prediction: if no test data are given, the
% function returns minus the log likelihood and its partial derivatives with
% respect to the hyperparameters; this mode is used to fit the hyperparameters.
% If test data are given, then (marginal) Gaussian predictions are computed,
% whose mean and variance are returned. Note that in cases where the covariance
% function has noise contributions, the variance returned in S2 is for noisy
% test targets; if you want the variance of the noise-free latent function, you
% must substract the noise variance.
%
% usage: [nlml dnlml] = gpr(logtheta, covfunc, x, y)
%    or: [mu S2]  = gpr(logtheta, covfunc, x, y, xstar)
%
% where:
%
%   logtheta is a (column) vector of log hyperparameters
%   covfunc  is the covariance function
%   x        is a n by D matrix of training inputs
%   y        is a (column) vector (of size n) of targets
%   xstar    is a nn by D matrix of test inputs
%   nlml     is the returned value of the negative log marginal likelihood
%   dnlml    is a (column) vector of partial derivatives of the negative
%                 log marginal likelihood wrt each log hyperparameter
%   mu       is a (column) vector (of size nn) of prediced means
%   S2       is a (column) vector (of size nn) of predicted variances
%
% For more help on covariance functions, see "help covFunctions".
%
% (C) copyright 2006 by Carl Edward Rasmussen (2006-03-20).


if ischar(covfunc), covfunc = cellstr(covfunc); end % convert to cell if needed

[n, D] = size(x{1});
trainSize = D;

lambda = logtheta(end);
logtheta = logtheta(1:end-1);

if eval(feval(covfunc{:})) ~= size(logtheta, 1)
  error('Error: Number of parameters do not agree with covariance function')
end

K1 = feval(covfunc{:}, logtheta, [x{1},x{2}]');    % compute training set covariance matrix
K11 = K1(1:trainSize, 1:trainSize);
K12 = K1(1:trainSize, trainSize+1:end)/(1+exp(lambda));
K21 = K12';
K22 = K1(trainSize+1:end, trainSize+1:end);
ys = y{1}';
yt = y{2}';

L11 = chol(K11)';

tmp = L11'\(L11\K12);
K = K22 - K21 * tmp;
beta = solve_chol(L11',ys);
miu = K21 * beta;

%L = chol(K)';                        % cholesky factorization of the covariance

% BUG FIX: check for non-positive definite
[L,p] = chol(K);                        % cholesky factorization of the covariance

% not positive definite?
if (p > 0)
  warning('kernel not positive definite (you might have repeating values!), regularizing...');
  
  % get smallest eigenvalue
  minE = min(eig(K));
  if (abs(minE) > 1e-6)
    minE
    error('min eig too big');
  end
  
  % regularize the kernel
  K = K + (1e-6)*eye(size(K));
  [L,p] = chol(K);
  if (p > 0)
    error('could not regularize kernel!');
  end
end

L = L';


if nargin == 4 % if no test cases, compute the negative log marginal likelihood
  alpha = solve_chol(L',yt-miu);
  
  
  out1 = 0.5*(yt-miu)'*alpha + sum(log(diag(L))) + 0.5*D*log(2*pi);
  
  if nargout == 2               % ... and if requested, its partial derivatives
    out2 = [zeros(size(logtheta));0];       % set the size of the derivative vector
    W = L'\(L\eye(length(yt)))-alpha*alpha';                % precompute for convenience
    for i = 1:length(out2)-1
      dK1 = feval(covfunc{:}, logtheta, [x{1},x{2}]', i);
      dK11 = dK1(1:trainSize, 1:trainSize);
      dK12 = dK1(1:trainSize, trainSize+1:end)/(1+exp(lambda));
      dK21 = dK12';
      dK22 = dK1(trainSize+1:end, trainSize+1:end);
      dC = dK22 - dK21*tmp - tmp'*dK12 + tmp'*dK11*tmp;
      dmiu = (dK21 + tmp'*dK11) * beta;
      out2(i) = sum(sum(W.*dC))/2 - dmiu'*alpha;
    end
    dC = 2*exp(lambda)*K21*tmp/(1+exp(lambda));
    dmiu = -exp(lambda)*miu/(1+exp(lambda));
    out2(end) = sum(sum(W.*dC))/2 - dmiu'*alpha;
  end
  
else                    % ... otherwise compute (marginal) test predictions ...
  
  [Kss, Kstar1] = feval(covfunc{:}, logtheta, x{1}', xstar);     %  test covariances
  [Kss, Kstar2] = feval(covfunc{:}, logtheta, x{2}', xstar);
  
  C = [K11, K12; K21, K22];
  LC = chol(C)';
  
  gamma = solve_chol(LC',[ys;yt]);
  Kstar = [Kstar1/(1+exp(lambda)); Kstar2];
  
  out1 = Kstar' * gamma;                                      % predicted means
  
  if nargout == 2
    v = LC\Kstar;
    out2 = Kss - sum(v.*v)';
  end
  
end
