function peoplecnt(opt)
% peoplecnt - feature extraction, training, prediction, and evaluation of people counting models.
%
%     peoplecnt(opt)
%
%    opt - options, see below:
%
% opt.mode      - 'feature', 'train', 'predict', 'eval'
% opt.recompute - recompute (do not use cached values)
% opt.update_flags - [feature, train, predict, eval]
% opt.debug     - debug mode
% opt.basedir   - base directory
%
% opt.imgnames{i}  - image file names
% opt.segmnames{i} - segmentation file names
% opt.segmmapname  - segmentation map
%
% opt.roiname      - ROI file name
% opt.pmapname     - perspective map file name
% opt.evalname     - evaluation set name
% opt.evalnum      - which CV set (e.g. 1, [1 2 3])
% opt.truthname{i} - ground-truth count file names
%
% opt.dirs        - directions (e.g. {'l', 'r'})
% opt.featpref{i} - prefix to save features (if '', do not save features)
% opt.feature{j}.name - feature name
% opt.feature{j}.opt  - feature options
% opt.feature{j}.sub  - select subset of features( [] = use all )
%
% opt.modelpref    - prefix to save model
% opt.model.name   - regression model name (e.g. 'lin', 'rvm', 'nn')
% opt.model.param  - regression parameters
% opt.model.norm   - normalize inputs/outputs
%                     'X' -- inputs: zero mean, unit variance
%                     'x' -- inputs: zero mean
% opt.predpref{i}  - prefix to save predictions
% opt.evalpref     - prefix to save evaluation
%
% opt.model_mismatch - mismatch directions (e.g. {'r', 'l'}) [optional]
%
% opt.updateflags - flags to force update on each stage, (e.g. [0 0 0 0])
% opt.flag_usemat - assume imgs and segms are .mat files (default=0, they are dat files)
% opt.flag_nogz   - do not gzip matlab file outputs (default=0, gzip output)
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

%%% add manual dependencies
%#function covConst covLINard covLINone covMatern3iso covMatern5iso
%#function covNNone covNoise covPeriodic covProd covRQard covRQiso
%#function covSEard covSEiso covSum covSEiso_fast covSEiso_fast2

fprintf('***** PEOPLECNT ********************************************\n');

if ~strcmp(opt.mode, {'feature', 'train', 'predict', 'eval'})
  error('bad mode!');
end

% setup default flags
if ~isfield(opt, 'flag_usemat');
  opt.flag_usemat = 0;
end
if ~isfield(opt, 'updateflags')
  opt.updateflags = [0 0 0 0];
end
if ~isfield(opt, 'flag_adapt');
  opt.flag_adapt = 0;
end
if ~isfield(opt, 'heavy');
  opt.heavy = 1;
end

olddir = pwd;
fprintf('- changing to %s\n', opt.basedir);
cd(opt.basedir);


%%% EXTRACT FEATURES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if any(strcmp(opt.mode, {'feature', 'train', 'predict', 'eval'}))
  f_all = peoplecnt_features(opt);
end


%%% TRAIN MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if any(strcmp(opt.mode, {'train', 'predict', 'eval'}))
  model_all = peoplecnt_train(f_all, opt);
end


%%% PREDICT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if any(strcmp(opt.mode, {'predict', 'eval'}))
  opt.flag_adapt = 1;
  opt.heavy = 1;
  if opt.heavy == 1 
    %% for 2000 frames
    %opt.train2 = [601 603 679 683 759 763 839 843 919 923 999 1003 1079 1083 1159 1163 1239 1243 1319 1323 1399 1400];
    %opt.train2 = [1 1 401 401 801 801 1201 1201 1601 1601];
    %opt.train2 = [1 1 201 201 401 401 601 601 801 801 1001 1001 1201 1201 1401 1401 1601 1601 1801 1801];
    %opt.train2 = [1 1 101 101 201 201 301 301 401 401 501 501 601 601 701 701 801 801 901 901 1001 1001 1101 1101 1201 1201 1301 1301 1401 1401 1501 1501 1601 1601 1701 1701 1801 1801 1901 1901];
    %opt.train2 = [1 3 101 102 201 203 301 302 401 403 501 502 601 603 701 702 801 803 901 902 1001 1003 1101 1102 1201 1203 1301 1302 1401 1403 1501 1502 1601 1603 1701 1702 1801 1803 1901 1902];
    %opt.train2 = [1 5 101 105 201 205 301 305 401 405 501 505 601 605 701 705 801 805 901 905 1001 1005 1101 1105 1201 1205 1301 1305 1401 1405 1501 1505 1601 1605 1701 1705 1801 1805 1901 1905];
    %% for 1000 frames
    opt.train2 = [1 5 101 105 201 205 301 305 401 405 501 505 601 605 701 705 801 805 901 905];
    %opt.train2 = [1 3 101 103 201 203 301 303 401 403 501 503 601 603 701 703 801 803 901 903];
    %opt.train2 = [1 1 101 101 201 201 301 301 401 401 501 501 601 601 701 701 801 801 901 901];
    %opt.train2 = [1 5 101 105 201 205 301 305];
    %opt.train2 = [1 20];
    %opt.train2 = [1 5];
  else
    opt.train2 = [1 5];
  end
  if opt.flag_adapt == 1
    [pred_all] = peoplecnt_predict_new(f_all, model_all, opt);
  else
    [pred_all] = peoplecnt_predict(f_all, model_all, opt);
  end
end


%%% EVALUATE MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if any(strcmp(opt.mode, {'eval'}))
  [eval_all] = peoplecnt_evaluate(pred_all, opt);
end

fprintf('- changing to %s\n', olddir);
cd(olddir);
