function [results_all] = peoplecnt_evaluate(pred_all, opt)
% peoplecnt_evaluate - evaluate predictions
%
%  [results_all] = peoplecnt_evaluate(pred_all, opt)
%
%                 opt - options structure
%      pred_all{d}{e} - predictions, for each direction d and evalset e
%   results_all{d}{e} - evaluation, for each direction d and evalset e
%
%  see peoplecnt for a description of the options.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

numdirs = length(opt.dirs);
numeval = length(opt.evalnum);
numimgs = length(opt.imgnames);

fprintf('=== peoplecnt_evaluate ====================================\n');


%%% load the evaluation set %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('loading evalset: %s\n', opt.evalname);
tmp = load(opt.evalname);
evalset = tmp.evalset;

results_all = cell(1,numdirs);


%%% FOR EACH DIRECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for dirsind = 1:numdirs
  mydir = opt.dirs{dirsind};
  
  cgt   = [];
  gtall = [];
  
  %%% FOR EACH EVALNUM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for evalind = 1:numeval
    myevalnum = opt.evalnum(evalind);
    
    % get the evaluation savefile
    if ~isempty(opt.evalpref)
      
      % change name for mismatch direction
      if isfield(opt, 'model_mismatch')
        fprintf('* model mismatch : using %s for %s\n', ...
          opt.model_mismatch{dirsind}, opt.dirs{dirsind});
        
        % add "mismatch-dir_"
        mystr = ['mismatch-' opt.model_mismatch{dirsind} '_' mydir];
      else
        
        % otherwise use direction code
        mystr = mydir;
      end
      
      outname = [opt.evalpref '_' mystr sprintf('_eval%d.mat', myevalnum)];
    else
      outname = '';
    end
    
    fprintf('*** dir=%s; eval=%d: file=%s ', mydir, myevalnum, outname);
    
    %%% check for cached evaluation
    if ~opt.recompute && ~opt.updateflags(4) && ~isempty(outname) && ...
        exist(outname, 'file')
      fprintf('[cached]\n');
      
      % load
      tmp = load(outname);
      results = tmp.results;
      
    else
      %%% EVALUATE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      fprintf('[evaluating]');
      
      % load ground-truth!
      if isempty(cgt)
        cgt = peoplecnt_loadgt(opt);
      end
      if isempty(gtall)
        [fvall, gtall] = peoplecnt_extractdata(opt, dirsind, [], cgt);
      end
      
      if isempty(gtall)
        error('found no ground-truth data!');
      end
      
      % concatenate predictions
      predall    = [];
      predvarall = [];
      for imgind = 1:numimgs
        predall    = [predall,    pred_all{dirsind}{evalind}{imgind}.pred];
        predvarall = [predvarall, pred_all{dirsind}{evalind}{imgind}.predvar];
      end
      
      % get the training/test sets
      trainset  = evalset{myevalnum}.trainset;
      trainset2 = evalset{myevalnum}.trainset2;
      testset   = evalset{myevalnum}.testset;
      
      fprintf('  - trainsize=%d (trainset2=%d); testsize=%d\n', length(trainset), ...
        length(trainset2), length(testset));
      
      %%% extract training and test set %%%
      predtrain  = predall(trainset);
      predtrain2 = predall(trainset2);
      predtest   = predall(testset);
      if ~isempty(predvarall)
        predvartrain  = predvarall(trainset);
        predvartrain2 = predvarall(trainset2);
        predvartest   = predvarall(testset);
      else
        predvartrain  = [];
        predvartrain2 = [];
        predvartest   = [];
      end
      cnttrain  = gtall(trainset);
      cnttrain2 = gtall(trainset2);
      cnttest   = gtall(testset);
      
      clear err perr mse pmse pcum pcum_bins Ebias pEbias Evar pEvar ...
        stats rstats abserr pabserr
      
      %%% evaluate model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      for i=1:4
        switch(i)
          case 1  % training
            myerr  = predtrain - cnttrain;
            myperr = round(predtrain) - cnttrain;
            myset = 'train';
          case 2  % training 2
            if (~isempty(predtrain2))
              myerr  = predtrain2 - cnttrain2;
              myperr = round(predtrain2) - cnttrain2;
            else
              myerr = [];
              myperr = [];
            end
            myset = 'train2';
          case 3  % testing
            myerr  = predtest  - cnttest;
            myperr = round(predtest) - cnttest;
            myset = 'test';
          case 4  % overall
            myerr  = [predall(1:length(gtall)) - gtall];
            myperr = [round(predall(1:length(gtall)))  - gtall];
            myset = 'all';
            
          otherwise
            error('bad i');
        end
        
        % check for isnan
        ii = find(isnan(myerr));
        if (length(ii) > 0)
          warning(sprintf('found %d NaN predictions', length(ii)));
          myerr(ii) = [];
          myperr(ii) = [];
        end
        
        eset{i} = myset;
        
        % prediction error
        err{i}  = myerr;
        perr{i} = myperr;
        
        % MSE and absolute error
        if isempty(myerr)
          mse(i)     = nan;
          pmse(i)    = nan;
          abserr(i)  = nan;
          pabserr(i) = nan;
        else
          mse(i)  = mean(myerr.^2);
          pmse(i) = mean(myperr.^2);
          abserr(i)  = mean(abs(myerr));
          pabserr(i) = mean(abs(myperr));
        end
        
        % cumulative error
        if (isempty(myperr))
          pcum{i} = [];
          pcum_bins{i} = [];
        else
          binmax = min(1000,max(abs(myperr)));
          [tmpn, tmpx] = hist(abs(myperr), 0:binmax);
          pcum{i}      = cumsum(tmpn) / sum(tmpn);
          pcum_bins{i} = tmpx;
        end
        
        % estimator bias and variance
        if (isempty(myerr))
          Ebias(i)  = nan;
          pEbias(i) = nan;
          Evar(i)   = nan;
          pEvar(i)  = nan;
        else
          Ebias(i)   = mean(myerr);
          pEbias(i)  = mean(myperr);
          Evar(i)    = var(myerr);
          pEvar(i)   = var(myperr);
        end
      end
      
      % statistics
      stats.set       = eset;
      stats.err       = err;
      stats.mse       = mse;
      stats.abserr    = abserr;
      stats.Ebias     = Ebias;
      stats.Evar      = Evar;
      
      % statistics (rounded predictions)
      rstats.set      = eset;
      rstats.err      = perr;
      rstats.mse      = pmse;
      rstats.abserr   = pabserr;
      rstats.cum      = pcum;
      rstats.cum_bins = pcum_bins;
      rstats.Ebias    = pEbias;
      rstats.Evar     = pEvar;
      
      stats
      rstats
      
      results.opt    = opt;
      results.dirsind = dirsind;
      results.evalind = evalind;
      results.stats  = stats;
      results.rstats = rstats;
      results.data.trainset      = trainset;
      results.data.trainset2     = trainset2;
      results.data.testset       = testset;
      results.data.predtrain     = predtrain;
      results.data.predtrain2    = predtrain2;
      results.data.predtest      = predtest;
      results.data.predvartrain  = predvartrain;
      results.data.predvartrain2 = predvartrain2;
      results.data.predvartest   = predvartest;
      results.data.cnttrain      = cnttrain;
      results.data.cnttrain2     = cnttrain2;
      results.data.cnttest       = cnttest;
      
      %%% save
      if ~isempty(outname)
        fprintf('[saved]\n');
        makedir(outname);
        save(outname, 'results');
      end
      
    end %end if cache
    
    % save to output
    if 1
      results_all{dirsind}{evalind} = results;
    end
    
  end % end for eval
  
  % average stats
  avgmse    = []; avgabserr = []; avgEbias  = []; avgEvar   = [];
  for i=1:numeval
    avgmse(end+1,:)    = results_all{dirsind}{i}.stats.mse;
    avgabserr(end+1,:) = results_all{dirsind}{i}.stats.abserr;
    avgEbias(end+1,:)  = results_all{dirsind}{i}.stats.Ebias;
    avgEvar(end+1,:)   = results_all{dirsind}{i}.stats.Evar;
  end
  results_avg{dirsind}.stats.mse    = mean(avgmse, 1);
  results_avg{dirsind}.stats.abserr = mean(avgabserr, 1);
  results_avg{dirsind}.stats.Ebias  = mean(avgEbias, 1);
  results_avg{dirsind}.stats.Evar   = mean(avgEvar, 1);
  
  avgmse    = []; avgabserr = []; avgEbias  = []; avgEvar   = [];
  for i=1:numeval
    avgmse(end+1,:)    = results_all{dirsind}{i}.rstats.mse;
    avgabserr(end+1,:) = results_all{dirsind}{i}.rstats.abserr;
    avgEbias(end+1,:)  = results_all{dirsind}{i}.rstats.Ebias;
    avgEvar(end+1,:)   = results_all{dirsind}{i}.rstats.Evar;
  end
  results_avg{dirsind}.rstats.mse    = mean(avgmse, 1);
  results_avg{dirsind}.rstats.abserr = mean(avgabserr, 1);
  results_avg{dirsind}.rstats.Ebias  = mean(avgEbias, 1);
  results_avg{dirsind}.rstats.Evar   = mean(avgEvar, 1);
  
  
end   % end for dir

%%% output %%%%
optdirs = opt.dirs;

fprintf('\n=== stats ===\n');
for dirsind = 1:numdirs
  fprintf('MSE-%s   ', optdirs{dirsind});
end
for dirsind = 1:numdirs
  fprintf('err-%s   ', optdirs{dirsind});
end
fprintf('\n');

for evalind = 1:numeval
  for dirsind = 1:numdirs
    fprintf('%0.4f  ', results_all{dirsind}{evalind}.rstats.mse(3));
  end
  for dirsind = 1:numdirs
    fprintf('%0.4f  ', results_all{dirsind}{evalind}.rstats.abserr(3));
  end
  fprintf('\n');
end

fprintf('\n=== avg stats ===\n');
for dirsind = 1:numdirs
  fprintf('MSE-%s   ', optdirs{dirsind});
end
for dirsind = 1:numdirs
  fprintf('err-%s   ', optdirs{dirsind});
end
fprintf('\n');
for dirsind = 1:numdirs
  fprintf('%0.4f  ', results_avg{dirsind}.rstats.mse(3));
end
for dirsind = 1:numdirs
  fprintf('%0.4f  ', results_avg{dirsind}.rstats.abserr(3));
end
fprintf('\n');
fprintf('\n');

%%%%%%%%%%%%%%%%%
