function f_all = peoplecnt_features(opt)
% peoplecnt_features - compute features
%
%    f = peoplecnt_features(opt)
%
%            opt - options structure
%    f_all{i}{d} - features for img i and direction d.
%
%  see peoplecnt for a description of the options.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

% 2008-09-01 - fixed bug with segmblobs: number of segments might not match true number of classes


%%% constants %%%
MINBLOBSIZE = 10;

fprintf('=== peoplecnt_features ====================================\n');

%%% LOAD METADATA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% load ROI %%%
fprintf('load roi: %s\n', opt.roiname);
roi     = load(opt.roiname);
roimask = roi.roi.mask;

%%% load perspective map %%%
fprintf('load pmap: %s\n', opt.pmapname);
tmp          = load(opt.pmapname);
pmap.pmapxy  = tmp.dmap.pmapxy;

% make sure the roimask is contained in pmapxy
ii = find(roimask);
if any(isnan(pmap.pmapxy(ii)))
  error('pmapxy does not contain all of roimask');
end
ii = find(isnan(pmap.pmapxy));
pmap.pmapxy(ii) = 0;
pmap.sqrtpmapxy = sqrt(pmap.pmapxy);

%%% load segmentation map %%%
fprintf('load smap: %s\n', opt.segmmapname);
smap    = load(opt.segmmapname);
smapK   = length(smap.segmmap);
fprintf('smap K=%d\n', smapK);

% map each direction
for ii=1:length(opt.dirs)
  segmind(ii) = -1;
  
  % find the segment index for this direction
  for i=1:smapK
    if (strcmp(smap.segmmap{i}, opt.dirs{ii}))
      segmind(ii) = i;
    end
  end
  
  % get the blob index for this direction
  switch(opt.dirs{ii})
    case  {'t'}
      blobind(ii) = smapK+1;
      segmind(ii) = 0;
    otherwise
      blobind(ii) = segmind(ii);
  end
  
  fprintf('mapping %s --> segment %d\n', opt.dirs{ii}, segmind(ii));
  
  if (segmind(ii) == -1)
    error('bad direction!');
  end
end


%%% allocate space for features %%%
f_all = cell(1,length(opt.imgnames));


%%% FOR EACH IMAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for imgind=1:length(opt.imgnames)
  
  %%% get names %%%
  myimgname   = opt.imgnames{imgind};
  mysegmname  = opt.segmnames{imgind};
  myfeatpref  = opt.featpref{imgind};
  if (isempty(myfeatpref))
    myblobsname = '';
  else
    myblobsname = [myfeatpref '.blobs.mat'];
  end
  
  fprintf('*** image: %s\n', myimgname);
  fprintf('*** segm:  %s\n', mysegmname);
  
  % clear the image caches
  img     = [];
  imgsize = [];
  segm    = [];
  blobs   = [];
  f_img   = cell(1, length(opt.dirs));
  
  %%% FOR EACH FEATURE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for featind=1:length(opt.feature)
    
    % get this feature
    myfeature = opt.feature{featind};
    
    %%% get feature size / output string %%%%%%%%%%%%
    switch(myfeature.name)
      case 'norm_area'
        fdim = 1;
        outsuf = 'na';
        
      case 'norm_perimeter'
        fdim = 1;
        outsuf = 'np';
        
      case 'norm_perimeterorient'
        fdim = myfeature.opt;
        outsuf = sprintf('npo%d', myfeature.opt(1));
        
      case 'norm_edge'
        fdim = 1;
        outsuf = 'ne';
        
      case 'norm_edgeorient'
        fdim = myfeature.opt;
        outsuf = sprintf('neo%d', myfeature.opt(1));
        
      case 'norm_glcm'
        fdim = 20;
        outsuf = 'nco';
        if (any(myfeature.opt == 's'))
          outsuf = [outsuf 's'];
        end
        if (any(myfeature.opt == 'f'))
          outsuf = [outsuf 'f'];
        end
        
      case 'norm_blobhist'
        fdim = myfeature.opt(1);
        outsuf = sprintf('nbh%d_%d', myfeature.opt(1), myfeature.opt(2));
        
      case 'ellipsoid'
        fdim = 6;
        outsuf = 'ell';
        
      case 'norm_paratio'
        fdim = 1;
        outsuf = 'npa';
        
      case 'norm_roundness'
        fdim = 1;
        outsuf = 'nr';
        
      case 'paratio'
        fdim = 1;
        outsuf = 'pa';
        
      case 'roundness'
        fdim = 1;
        outsuf = 'r';
        
      case 'norm_edgeminkowski'
        fdim = 1;
        outsuf = 'nem';
        
      case 'norm_perimeterminkowski'
        fdim = 1;
        outsuf = 'npm';
        
      case 'numblobs'
        fdim = 1;
        outsuf = 'nb';
        
      otherwise
        error('bad feature');
    end
    
    %%% FOR EACH DIRECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for dirsind = 1:length(opt.dirs)
      
      mydir = opt.dirs{dirsind};
      
      %%% build output string %%%%%%%%%%%%%%%%%%%%%%%
      if isempty(myfeatpref)
        myoutname = '';
      else
        myoutname = [myfeatpref '.' outsuf '.' mydir '.mat'];
      end
      
      fprintf('     - (%s-%s) file=%s ', myfeature.name, mydir, myoutname);
      
      %%% try to load cached features %%%%%%%%%%%%%%%%%%%%%%
      if ~opt.recompute && ~opt.updateflags(1) && ~isempty(myoutname) && ...
          exist(myoutname, 'file')
        fprintf('[cached]\n');
        tmp = load(myoutname);
        
        % check size
        if (fdim ~= size(tmp.fv.f, 1))
          error('bad feature dimension in cached file.');
        end
        
        % get feature
        f = tmp.fv.f;
        
      else
        %%% compute the feature %%%%%%%%%%%%%%%%%%%%%%%%%%%%
        fprintf('[extracting]');
        
        % load images, if not yet loaded
        if (isempty(img))
          if (~opt.flag_usemat)
            img     = loaddat(myimgname, 't');
            segm    = loaddat(mysegmname, 't');
          else
            tmp  = load(myimgname);
            img  = tmp.img;
            tmp  = load(mysegmname);
            segm = tmp.segm;
          end
          
          imgsize = size(img);
          if (ndims(imgsize) > 3)
            error('does not support color images!\n');
          end
        end
        
        % make blobs, for certain features
        if any(strcmp(myfeature.name, {'norm_blobhist', 'numblobs'}))
          if isempty(blobs)
            blobs = peoplecnt_segmblobs(segm, smapK, myblobsname, ...
              opt.recompute || opt.updateflags(1), ...
              opt.flag_nogz);
          end
        end
        
        % preallocate f
        f = zeros(fdim, imgsize(3));
        
        %%% COMPUTE FEATURE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for t=1:imgsize(3)
          % get image
          myimg  = double(img(:,:,t));
          
          % get segmentation for this direction
          if (segmind(dirsind) > 0)
            mysegm  = (segm(:,:,t) == segmind(dirsind));
          elseif (segmind == 0) % get everything
            mysegm  = (segm(:,:,t) >= 1);
          else
            error('bad segmind');
          end
          
          mydebug = 0;
          
          % compute feature
          switch(myfeature.name)
            case 'norm_area'
              myf = feature_norm_area(myimg, mysegm, pmap, roimask, myfeature.opt, ...
                mydebug);
              
            case 'norm_perimeter'
              myf = feature_norm_perimeter(myimg, mysegm, pmap, roimask, myfeature.opt, ...
                mydebug);
              
            case 'norm_perimeterorient'
              myf = feature_norm_perimeterorient(myimg, mysegm, pmap, roimask, myfeature.opt, ...
                mydebug);
              
            case 'norm_edge'
              myf = feature_norm_edge(myimg, mysegm, pmap, roimask, myfeature.opt, ...
                mydebug);
              
            case 'norm_edgeorient'
              myf = feature_norm_edgeorient(myimg, mysegm, pmap, roimask, myfeature.opt, ...
                mydebug);
              
            case 'norm_glcm'
              myf = feature_norm_glcm(myimg, mysegm, pmap, roimask, myfeature.opt, ...
                mydebug);
              
            case 'norm_blobhist'
              myblobs = blobs{blobind(dirsind)}(:,:,t);
              myf = feature_norm_blobhist(myimg, myblobs, pmap, roimask, ...
                myfeature.opt, mydebug);
              
            case 'ellipsoid'
              myf = feature_ellipsoid(myimg, mysegm, pmap, roimask, ...
                myfeature.opt, mydebug);
              
            case 'norm_paratio'
              myf = feature_norm_paratio(myimg, mysegm, pmap, roimask, ...
                myfeature.opt, mydebug);
              
            case 'norm_roundness'
              myf = feature_norm_roundness(myimg, mysegm, pmap, roimask, ...
                myfeature.opt, mydebug);
              
            case 'paratio'
              myf = feature_paratio(myimg, mysegm, pmap, roimask, ...
                myfeature.opt, mydebug);
              
            case 'roundness'
              myf = feature_roundness(myimg, mysegm, pmap, roimask, ...
                myfeature.opt, mydebug);
              
            case 'norm_edgeminkowski'
              myf = feature_norm_edgeminkowski(myimg, mysegm, pmap, roimask, myfeature.opt, ...
                mydebug);
              
            case 'norm_perimeterminkowski'
              myf = feature_norm_perimeterminkowski(myimg, mysegm, pmap, roimask, myfeature.opt, ...
                mydebug);
              
            case 'numblobs'
              myblobs = blobs{blobind(dirsind)}(:,:,t);
              myf = feature_numblobs(myimg, myblobs, pmap, roimask, myfeature.opt, mydebug);
              
            otherwise
              error('bad feature name');
          end
          
          % save feature in array
          f(:,t) = myf;
        end % end for t
        
        %%% save feature to disk %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if ~isempty(myoutname)
          fprintf('[saving]');
          fv.f   = f;
          fv.opt = opt;
          fv.imgind  = imgind;
          fv.featind = featind;
          fv.dirsind = dirsind;
          makedir(myoutname);
          save(myoutname, 'fv');
        end
        fprintf('\n');
        
      end % end cache/no cache
      
      %%% extract subset %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      if (~isempty(myfeature.sub))
        f = f(myfeature.sub,:);
      end
      
      %%% append feature %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      f_img{dirsind} = [f_img{dirsind}; f];
      
    end % end dirsind
  end % end featind
  
  % save features for this image
  f_all{imgind} = f_img;
  
end % end imgind
