function [pred_all] = peoplecnt_predict_new(f_all, model_all, opt);
% peoplecnt_predict - make count predictions
%
%    [pred_all] = peoplecnt_predict_new(f_all, model_all,opt);
%
%                opt - options structure
%        f_all{i}{d} - features for img i and direction d.
%    model_all{d}{e} - model for direction d and evalset e
%  pred_all{d}{e}{i} - predictions for directiond, evalset e, and image i.
%
%  see peoplecnt for a description of the options.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

numdirs = length(opt.dirs);
numeval = length(opt.evalnum);
numimgs = length(opt.imgnames);

fprintf('=== peoplecnt_predict ====================================\n');

pred_all = cell(1,numdirs);


%%% FOR EACH DIRECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for dirsind = 1:numdirs
  mydir = opt.dirs{dirsind};
  
  pred_all{dirsind} = cell(1,numeval);
  
  %%% FOR EACH EVALNUM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for evalind = 1:numeval
    myevalnum = opt.evalnum(evalind);
    
    pred_all{dirsind}{evalind} = cell(1,numimgs);
    
    %%% FOR EACH IMAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for imgind = 1:numimgs
      
      % get the prediction file
      if ~isempty(opt.predpref)
        
        % change name for mismatch direction
        if isfield(opt, 'model_mismatch')
          fprintf('* model mismatch : using %s for %s\n', ...
            opt.model_mismatch{dirsind}, opt.dirs{dirsind});
          
          % add "mismatch-dir_"
          mystr = ['mismatch-' opt.model_mismatch{dirsind} '_' mydir];
        else
          
          % otherwise use direction code
          mystr = mydir;
        end
        
        outname = [opt.predpref{imgind} '_' mystr sprintf('_eval%d.mat', myevalnum)];
      else
        outname = '';
      end
      
      fprintf('*** dir=%s; eval=%d; img=%d: file=%s ', mydir, ...
        myevalnum, imgind, outname);
      
      %%% check for cached prediction %%%
      if ~opt.recompute && ~opt.updateflags(3) && ~isempty(outname) && ...
          exist(outname, 'file')
        fprintf('[cached]\n');
        
        % load
        tmp = load(outname);
        pred = tmp.pred;
        
      else
        
        %%% COMPUTE PREDICTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        fprintf('[predicting]');
        
        % get feature-vectors
        fvall = f_all{imgind}{dirsind};
        
        % get model
        mymodel = model_all{dirsind}{evalind};
        
        
        %%% normalize %%%%%%%%%%%%%%%%%%%%%%%%%
        xmean = mymodel.xmean;
        xstd  = mymodel.xstd;
        
        % check normalization
        if any(opt.model.norm == 'X')
          fprintf('  - normalizing (0,1); ');
          if (isempty(xmean) || isempty(xstd))
            error('normalization mismatch');
          end
        elseif any(opt.model.norm == 'x')
          fprintf('  - normalizing mean; ');
          if (isempty(xmean) || ~isempty(xstd))
            error('normalization mismatch');
          end
        else
          fprintf('  - no normalization; ');
          if (~isempty(xmean) || ~isempty(xstd))
            error('normalization mimsatch');
          end
        end
        
        % normalize
        if ~isempty(xmean)
          fvall = fvall - repmat(xmean, 1, size(fvall,2));
        end
        if ~isempty(xstd)
          fvall = fvall ./ repmat(xstd, 1, size(fvall,2));
        end
        
        
        %%% predict model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% gaussian process %%%%%%%%%%%%%%%%
        
        if dirsind == 1
          [mypred, mypredvar] = peoplecnt_gp_predict(fvall, mymodel.model, opt, f_all, xmean, xstd);
        else
          [mypred, mypredvar] = gp_predict(fvall, mymodel.model);
        end
        mypred  = mypred(:)';
        mypredvar = mypredvar(:)';
        
        
        
        mypred    = mypred(:)';
        mypredvar = mypredvar(:)';
        
        pred.opt     = opt;
        pred.imgind  = imgind;
        pred.dirsind = dirsind;
        pred.evalind = evalind;
        pred.pred    = max(mypred,0);
        pred.predvar = mypredvar;
        pred.predraw = [];
        
        % save
        if ~isempty(outname)
          fprintf('[saved]');
          makedir(outname);
          save(outname, 'pred');
        end
        
        fprintf('\n');
        
      end % end check cache
      
      % save to output
      pred_all{dirsind}{evalind}{imgind} = pred;
      
      
    end % end for img
    
  end % end for evalnum
end % end for dir

