function model_all = peoplecnt_train(f_all, opt)
% peoplecnt_train - train a counting model
%
%   model_all = peoplecnt_train(f_all, opt)
%
%                opt - options structure
%        f_all{i}{d} - features for img i and direction d.
%    model_all{d}{e} - model for direction d and evalset e
%
%  see peoplecnt for a description of the options.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

numdirs = length(opt.dirs);
numeval = length(opt.evalnum);

fprintf('=== peoplecnt_train ====================================\n');


%%% load the evaluation set %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('loading evalset: %s\n', opt.evalname);
tmp = load(opt.evalname);
evalset = tmp.evalset;

model_all = cell(1,numdirs);

cgt   = [];

%%% FOR EACH DIRECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for dirsind = 1:numdirs
  % use model mismatch direction
  if isfield(opt, 'model_mismatch')
    fprintf('* model mismatch : using %s for %s\n', ...
      opt.model_mismatch{dirsind}, opt.dirs{dirsind});
    mydir = opt.model_mismatch{dirsind};
  else
    % use normal direction
    mydir = opt.dirs{dirsind};
  end
  
  fvall = [];
  gtall = [];
  
  %%% FOR EACH EVALNUM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for evalind = 1:numeval
    myevalnum = opt.evalnum(evalind);
    
    % get the model savefile
    if ~isempty(opt.modelpref)
      outname = [opt.modelpref '_' mydir sprintf('_eval%d.mat', myevalnum)];
    else
      outname = '';
    end
    
    fprintf('*** dir=%s - eval=%d: file=%s ', mydir, myevalnum, outname);
    
    %%% check for cached model
    if ~opt.recompute && ~opt.updateflags(2) && ~isempty(outname) && ...
        exist(outname, 'file')
      fprintf('[cached]\n');
      
      % load
      tmp = load(outname);
      model = tmp.model;
      
    else
      %%% TRAIN MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      fprintf('[training]\n');
      
      if isempty(cgt)
        cgt = peoplecnt_loadgt(opt);
      end
      
      % load data
      if isempty(fvall)
        [fvall, gtall] = peoplecnt_extractdata(opt, dirsind, f_all, cgt);
      end
      
      if (isempty(gtall))
        error('found no ground-truth data!');
      end
      
      % get the training set
      trainset  = evalset{myevalnum}.trainset;
      
      fprintf('  - trainsize=%d\n', length(trainset));
      
      % extract training and test set
      if opt.heavy == 1
        trainset = [1:3,101:102,201:203,301:302,401:403,501:502,601:603,701:702,801:803,901:902,1001:1003,1101:1102,1201:1203,1301:1302,1401:1403,1501:1502,1601:1603,1701:1702,1801:1803,1901:1902];
        %trainset = [1:5,101:105,201:205,301:305,401:405,501:505,601:605,701:705,801:805,901:905];
        %trainset = [1:50];
        %trainset = [1:5,101:105,201:205,301:305];
        %trainset = [1:5];
      else
        trainset = [1:5];
      end
      tmpfeat = load(opt.sourse);
      fvtrain   = {tmpfeat.features(:,601:1400), fvall(:,trainset)};
      cnttrain  = {tmpfeat.y(601:1400)', gtall(trainset)};
      
      fprintf('  - feature size = %d\n', size(fvtrain,1));
      
      %%% normalize %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      if any(opt.model.norm == 'X')
        fprintf('  - normalizing (0,1)\n');
        xmean = mean([fvtrain{1}, fvtrain{2}],2);
        xstd  = std([fvtrain{1},fvtrain{2}]')';
        
        % check for xstd=0
        ii = find(xstd == 0);
        if ~isempty(ii)
          fprintf('*** WARNING: these features are empty: ');
          fprintf('%d ', ii);
          fprintf('\n');
          % force no scaling
          xstd(ii) = 1;
        end
      elseif any(opt.model.norm == 'x')
        fprintf('  - normalizing mean\n');
        xmean = mean([fvtrain{1}, fvtrain{2}],2);
        xstd  = [];
      else
        fprintf('  - no normalization\n');
        xmean = [];
        xstd  = [];
      end
      
      % normalize mean
      if ~isempty(xmean)
        fvtrain{1} = fvtrain{1} - repmat(xmean, 1, size(fvtrain{1},2));
        fvtrain{2} = fvtrain{2} - repmat(xmean, 1, size(fvtrain{2},2));
      end
      
      % normalize std
      if ~isempty(xstd)
        fvtrain{1} = fvtrain{1} ./ repmat(xstd, 1, size(fvtrain{1},2));
        fvtrain{2} = fvtrain{2} ./ repmat(xstd, 1, size(fvtrain{2},2));
      end
      
      
      %%% clean data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % remove points that are the same (this happens when scene is empty)
      [tmp, tmpi, tmpj] = unique(fvtrain{1}', 'rows');  % get unique rows
      if (length(tmpi) < length(cnttrain{1}))
        fvtrain_uniq1  = tmp';
        cnttrain_uniq1 = cnttrain{1}(tmpi);   % extract counts
        fprintf('cleaning data: found %d/%d unique points\n', length(cnttrain_uniq1), length(cnttrain));
      else
        fvtrain_uniq1 = fvtrain{1};
        cnttrain_uniq1 = cnttrain{1};
        fprintf('cleaning data: ok\n');
      end
      
      [tmp, tmpi, tmpj] = unique(fvtrain{2}', 'rows');  % get unique rows
      if (length(tmpi) < length(cnttrain{2}))
        fvtrain_uniq2  = tmp';
        cnttrain_uniq2 = cnttrain{2}(tmpi);   % extract counts
        fprintf('cleaning data: found %d/%d unique points\n', length(cnttrain_uniq1), length(cnttrain));
      else
        fvtrain_uniq2 = fvtrain{2};
        cnttrain_uniq2 = cnttrain{2};
        fprintf('cleaning data: ok\n');
      end
      fvtrain_uniq = {fvtrain_uniq1, fvtrain_uniq2};
      cnttrain_uniq = {cnttrain_uniq1, cnttrain_uniq2};
      
      fprintf('model: %s\n', opt.model.name);
      
      %%% learn model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %%% gaussian process %%%%%%%%%%%%%%%%
      fprintf('covfunc: %s\n', celldispstr(opt.model.param.covfunc));
      fprintf('normmode: %s\n', opt.model.param.normmode);
      fprintf('numtrials: %d\n', opt.model.param.numtrials);
      if dirsind==1
        regmodel = gp_train_new(fvtrain_uniq, cnttrain_uniq, opt.model.param.covfunc, ...
          opt.model.param.normmode, opt.model.param.numtrials);
      else
        regmodel = gp_train(fvtrain_uniq1, cnttrain_uniq1, opt.model.param.covfunc, ...
          opt.model.param.normmode, opt.model.param.numtrials);
      end
      
      % put model into structure
      model.opt  = opt;
      model.xmean = xmean;
      model.xstd  = xstd;
      model.model = regmodel;
      model.dirsind = dirsind;
      model.evalind = evalind;
      
      % save
      if (~isempty(outname))
        fprintf('[saved]\n');
        makedir(outname);
        save(outname, 'model');
      end
      
    end % check cache
    
    % accumulate results
    model_all{dirsind}{evalind} = model;
    
  end % end for evalnum
end % end for dirs


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function str = celldispstr(c)
% celldispstr - display cell strings

str = '{';

for ci = 1:length(c)
  if (ci > 1)
    str = [str ', '];
  end
  if (iscell(c{ci}))
    str = [str celldispstr(c{ci})];
  else
    str = [str '''' c{ci} ''''];
  end
end
str = [str '}'];

