function model_all = peoplecnt_train(f_all, opt)
% peoplecnt_train - train a counting model
%
%   model_all = peoplecnt_train(f_all, opt)
%
%                opt - options structure
%        f_all{i}{d} - features for img i and direction d.
%    model_all{d}{e} - model for direction d and evalset e
%
%  see peoplecnt for a description of the options.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

numdirs = length(opt.dirs);
numeval = length(opt.evalnum);

fprintf('=== peoplecnt_train ====================================\n');


%%% load the evaluation set %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('loading evalset: %s\n', opt.evalname);
tmp = load(opt.evalname);
evalset = tmp.evalset;

model_all = cell(1,numdirs);

cgt   = [];

%%% FOR EACH DIRECTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for dirsind = 1:numdirs
  % use model mismatch direction
  if isfield(opt, 'model_mismatch')
    fprintf('* model mismatch : using %s for %s\n', ...
      opt.model_mismatch{dirsind}, opt.dirs{dirsind});
    mydir = opt.model_mismatch{dirsind};
  else
    % use normal direction
    mydir = opt.dirs{dirsind};
  end
  
  fvall = [];
  gtall = [];
  
  %%% FOR EACH EVALNUM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  for evalind = 1:numeval
    myevalnum = opt.evalnum(evalind);
    
    % get the model savefile
    if ~isempty(opt.modelpref)
      outname = [opt.modelpref '_' mydir sprintf('_eval%d.mat', myevalnum)];
    else
      outname = '';
    end
    
    fprintf('*** dir=%s - eval=%d: file=%s ', mydir, myevalnum, outname);
    
    %%% check for cached model
    if ~opt.recompute && ~opt.updateflags(2) && ~isempty(outname) && ...
        exist(outname, 'file')
      fprintf('[cached]\n');
      
      % load
      tmp = load(outname);
      model = tmp.model;
      
    else
      %%% TRAIN MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      fprintf('[training]\n');
      
      if isempty(cgt)
        cgt = peoplecnt_loadgt(opt);
      end
      
      % load data
      if isempty(fvall)
        [fvall, gtall] = peoplecnt_extractdata(opt, dirsind, f_all, cgt);
      end
      
      if (isempty(gtall))
        error('found no ground-truth data!');
      end
      
      % get the training set
      tmpfeat = load(opt.sourse);
      tmppairs = load(opt.pairs);
      fvtrain   = [tmpfeat.features(:,601:1400).*repmat(tmppairs.index.beta,1,800), fvall(:,tmppairs.index.target)];
      cnttrain  = [tmpfeat.y(601:1400)', gtall(tmppairs.index.target)];
      
      fprintf('  - feature size = %d\n', size(fvtrain,1));
      
      %%% normalize %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      if any(opt.model.norm == 'X')
        fprintf('  - normalizing (0,1)\n');
        xmean = mean(fvtrain,2);
        xstd  = std(fvtrain')';
        
        % check for xstd=0
        ii = find(xstd == 0);
        if ~isempty(ii)
          fprintf('*** WARNING: these features are empty: ');
          fprintf('%d ', ii);
          fprintf('\n');
          % force no scaling
          xstd(ii) = 1;
        end
      elseif any(opt.model.norm == 'x')
        fprintf('  - normalizing mean\n');
        xmean = mean(fvtrain,2);
        xstd  = [];
      else
        fprintf('  - no normalization\n');
        xstd  = [];
        xmean = [];
      end
      
      % normalize mean
      if ~isempty(xmean)
        fvtrain = fvtrain - repmat(xmean, 1, size(fvtrain,2));
      end
      
      % normalize std
      if ~isempty(xstd)
        fvtrain = fvtrain ./ repmat(xstd, 1, size(fvtrain,2));
      end
      
      
      %%% clean data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % remove points that are the same (this happens when scene is empty)
      [tmp, tmpi, tmpj] = unique(fvtrain', 'rows');  % get unique rows
      if (length(tmpi) < length(cnttrain))
        fvtrain_uniq  = tmp';
        cnttrain_uniq = cnttrain(tmpi);   % extract counts
        fprintf('cleaning data: found %d/%d unique points\n', length(cnttrain_uniq), length(cnttrain));
      else
        fvtrain_uniq = fvtrain;
        cnttrain_uniq = cnttrain;
        fprintf('cleaning data: ok\n');
      end
      
      fprintf('model: %s\n', opt.model.name);
      
      %%% learn model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %%% gaussian process %%%%%%%%%%%%%%%%
      fprintf('covfunc: %s\n', celldispstr(opt.model.param.covfunc));
      fprintf('normmode: %s\n', opt.model.param.normmode);
      fprintf('numtrials: %d\n', opt.model.param.numtrials);
      regmodel = gp_train(fvtrain_uniq, cnttrain_uniq, opt.model.param.covfunc, ...
        opt.model.param.normmode, opt.model.param.numtrials);
      
      % put model into structure
      model.opt  = opt;
      model.xmean = xmean;
      model.xstd  = xstd;
      model.model = regmodel;
      model.dirsind = dirsind;
      model.evalind = evalind;
      
      % save
      if (~isempty(outname))
        fprintf('[saved]\n');
        makedir(outname);
        save(outname, 'model');
      end
      
    end % check cache
    
    % accumulate results
    model_all{dirsind}{evalind} = model;
    
  end % end for evalnum
end % end for dirs


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function str = celldispstr(c)
% celldispstr - display cell strings

str = '{';

for ci = 1:length(c)
  if (ci > 1)
    str = [str ', '];
  end
  if (iscell(c{ci}))
    str = [str celldispstr(c{ci})];
  else
    str = [str '''' c{ci} ''''];
  end
end
str = [str '}'];

