% gp_demo - demo for GP regression
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

clear
close all

% reset random number generator
rand('state',18);
randn('state',20);

% setup covariance function
covfunc = {'covSum', {'covSEiso','covNoise'}};

% generate 1D data
loghyper = [log(1.0); log(1.0); log(0.1)];
n = 20;
x = 15*(rand(n,1)-0.5);
y = chol(feval(covfunc{:}, loghyper, x))'*randn(n,1);  
x = x';
y = y';

% learn GP
gp = gp_train(x, y, covfunc);

% predict GP
xstar = linspace(-7.5, 7.5, 201);
[ystar, sstar] = gp_predict(xstar, gp);

% plot results
sstar = sstar - exp(2*loghyper(3));
plot(xstar, ystar, 'b-', 'LineWidth', 2);
hold on
plot(xstar, ystar+2*sqrt(sstar), 'b-');
plot(xstar, ystar-2*sqrt(sstar), 'b-');
plot(x, y, 'k+', 'MarkerSize', 17)       
hold off
grid

% generate 2D data
mu1 = [-5; 0];
cv1 = [1;1];
mu2 = [3; 3];
cv2 = [2;2];
mu3 = [0;0];
cv3 = [3;3];

% sample points on grid
X1 = repmat(mu1,[1 50]) + diag(sqrt(cv1))*randn(2,50);
X2 = repmat(mu2,[1 50]) + diag(sqrt(cv2))*randn(2,50);
X3 = repmat(mu3,[1 50]) + diag(sqrt(cv3))*randn(2,50);
X = [X1, X2, X3];
Y = zeros(1,size(X,2));

% generate function values
tmp = X-repmat(mu1,[1 length(Y)]);
Y1 = exp(-0.5*sum(tmp.*(diag(cv1)*tmp),1))/(2*pi*sqrt(prod(cv1)));
tmp = X-repmat(mu2,[1 length(Y)]);
Y2 = exp(-0.5*sum(tmp.*(diag(cv2)*tmp),1))/(2*pi*sqrt(prod(cv2)));
tmp = X-repmat(mu3,[1 length(Y)]);
Y3 = exp(-0.5*sum(tmp.*(diag(cv3)*tmp),1))/(2*pi*sqrt(prod(cv3)));
Y = Y1+Y2+Y3;

% add noise
T = Y + 0.1*randn(1,length(Y));

% train GP
gp = gp_train(X, T, covfunc, 'X');

% predict on grid
xgrid = -10:0.5:10;
ygrid = -10:0.5:10;
[tmpx, tmpy] = meshgrid(xgrid, ygrid);
XX = [tmpx(:) tmpy(:)]';
[y_gp, s_gp] = gp_predict(XX, gp);

% view results
figure
subplot(1,3,1)
plot(X(1,:), X(2,:), 'k.');
title('sample points');
grid
subplot(1,3,2)
surf(xgrid, ygrid, reshape(y_gp, length(xgrid), length(ygrid)));
title('GP mean');
subplot(1,3,3)
surf(xgrid, ygrid, reshape(sqrt(s_gp), length(xgrid), length(ygrid)));
title('GP stddev');
