function [L, param] = gp_labelhyper(covfunc, logtheta, D)
% gp_labelhyper - label the log-hyperparameters, and convert to natural form.
%
%    [L, param] = gp_labelhyper(covfunc, logtheta, D)
%
%     INPUT
%   covfunc = the covariance function
%  logtheta = the log-hyperparameters
%         D = dimension of the data
%
%    OUTPUT
%         L = the label for each parameter
%     param = the parameter
%
% This is a list of the covariance functions and their log-parameters and
% natural parameters.
%
% name       logtheta/natural              k(x^p,x^q)
% ----------------------------------------------------
% covConst   [log(sqrt(s2))]               1/s2
%            C=1/sqrt(s2)                  C^2
%
% covLINard  [log(L(1:D))]                 x^p'*inv(P)*x^q,         P = diag(L(1:D)^2)
%            l=1/L                         (x^p.*l)'*(x^q.*l)
%
% covLINone  [log(sqrt(t2))]               x^p'*inv(P)*x^q + 1/t2,  P=t2*I
%            L=1/sqrt(t2)                  L^2*(x^p'*x^q + 1)
%
% covNoise   [log(sqrt(s2))]               s2*delta(p,q)
%            N=sqrt(s2)                    N^2 delta(p,q)
%
% covSEard   [log(L(1:D)) log(sqrt(sf2))]  sf2*exp(-(x^p-x^q)'*inv(P)*(x^p-x^q)/2), P=diag(L(1:D)^2)
%            l=1/L, R=sqrt(sf2)            R^2*exp(-0.5*((x^p-x^q).*l)^2)
%
% covSEiso   [log(L) log(sqrt(sf2))]       sf2*exp(-(x^p-x^q)'*inv(P)*(x^p-x^q)/2), P=L^2 *I
%             L, R=sqrt(sf2)               R^2*exp(-0.5*((x^p-x^q)/L)^2)
%
% covMatern3iso [log(L), log(sqrt(sf2)]    sf2*(1 + sqrt(3)*d(x^p,x^q)) * exp(-sqrt(3)*d(x^p,x^q))
%                                             d=sqrt((x^p-x^q)'*inv(P)*(x^p-x^q)), P = L^2*I ?
%              l=1/L, R=sqrt(sf2)          R^2*...
%                                             d=sqrt(((x^p-x^q).*l))
%
% covMatern5iso (same as covMatern3iso)
%
% covRQiso   [log(L) log(sqrt(sf2))        sf2*[1+(x^p-x^q)'*inv(P)*(x^p-x^q)/(2*alpha)]^(-alpha)
%             log(alpha)]                    P = L^2 *I
%            l=1/L, R=sqrt(sf2)        
%
% covPoly2iso [log(sqrt(t2))]              (x^p'*inv(P)*x^q + 1./t2)^2
%             L=1/sqrt(t2)
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

covflat = cellflatten(covfunc);

L     = {};
param = [];
ind   = 1;

for i=1:length(covflat)
  switch(covflat{i})
   case {'covSum', 'covProd'}
    % nothing
    
   case 'covConst'
    L{end+1}     = 'C';
    param(end+1) = exp(-logtheta(ind));
    ind = ind+1;
   
   case 'covLINard'
    for j=1:D
      L{end+1}     = ['Ll_' sprintf('%d', j)];
      param(end+1) = exp(-logtheta(ind));
      ind = ind+1;
    end
    
   case 'covLINone'
    L{end+1}     = 'L';
    param(end+1) = exp(-logtheta(ind));
    ind = ind+1;
    
   case 'covNoise'
    L{end+1}     = 'N';
    param(end+1) = exp(logtheta(ind));
    ind = ind+1;
    
   case 'covSEard'
    for j=1:D
      L{end+1}     = ['Rl_' sprintf('%d', j)];
      param(end+1) = exp(-logtheta(ind));
      ind = ind+1;
    end
    L{end+1}     = 'R';
    param(end+1) = exp(logtheta(ind));
    ind = ind+1;    
    
   case {'covSEiso', 'covSEiso_fast', 'covSEiso_fast2'}
    L{end+1}     = 'RL';
    tmpL         = exp(logtheta(ind));
    param(end+1) = tmpL;
    ind = ind+1;
    L{end+1}     = 'R';
    param(end+1) = exp(logtheta(ind));
    %sf2          = exp(2*logtheta(ind));
    %param(end+1) = sf2*((2*pi)^(D/2))*(tmpL^D);
    ind = ind+1;   
    
   case 'covMatern3iso'
    L{end+1}     = 'M3l';
    param(end+1) = exp(-logtheta(ind));
    ind = ind+1;
    L{end+1}     = 'M3';
    param(end+1) = exp(logtheta(ind));
    ind = ind+1;  
    
   case 'covMatern5iso'
    L{end+1}     = 'M5l';
    param(end+1) = exp(-logtheta(ind));
    ind = ind+1;
    L{end+1}     = 'M5';
    param(end+1) = exp(logtheta(ind));
    ind = ind+1;  
      
   case 'covRQiso'
    L{end+1}     = 'Ql';
    param(end+1) = exp(-logtheta(ind));
    ind = ind+1;
    L{end+1}     = 'Q';
    param(end+1) = exp(logtheta(ind));
    ind = ind+1;  
    L{end+1}     = 'Qa';
    param(end+1) = exp(logtheta(ind));
    ind = ind+1;   
    
   case 'covPoly2iso'
    L{end+1}     = 'L';
    param(end+1) = exp(-logtheta(ind));
    ind = ind+1;
    
   otherwise
    error('cov type not supported yet');
  end  
end


if (ind <= length(logtheta))
  error('index mismatch');
end

