function gp = gp_train(X, Y, covfunc, normmode, numtrials)
% gp_train - train a Gaussian Process regression function
%
%    gp = gp_train(X, Y, covfunc, normmode, numtrials)
% 
%         X = feature vectors (each column is a feature vector)
%         Y = function values
%   covfunc = covariance function ( default = {'covSum',{'covSEiso','covNoise'}} );
%  normmode = normalization mode:
%             'y' - normalize Y to zero-mean
%             'x' - normalize X to zero-mean
%             'X' - normalize X to zero-mean, unit variance
% numtrials = number of random initializations (the first is always fixed)
%           = (if negative, then all are random)
%
%  Train a regression function Y=f(X) using GP.  See "help gp_kernels" for
%  more covariance functions. 
%
% This uses the GPML library by C. E. Rasmussen.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

if (nargin < 3)
  covfunc = {'covSum', {'covSEiso','covNoise'}};
end
if (nargin < 4)
  normmode = '';
end
if (nargin < 5)
  numtrials = 1;
end

MAXIT = 500;

% Normalize the inputs
if any(normmode=='x') || any(normmode == 'X')
  Xmean = mean(X,2);
  X = X - repmat(Xmean,1,size(X,2));
  gp.Xmean = Xmean;
  
  if any(normmode == 'X')
    Xstd = std(X,0,2);
    X = X./repmat(Xstd,1,size(X,2));
    gp.Xstd = Xstd;
  end  
end
% normalize output
if any(normmode=='y')
  Ymean = mean(Y);   
  Y = Y - Ymean;    % subtract mean from Y
  gp.Ymean    = Ymean;
end


% learn the GP
D = size(X,1);
nump = eval(feval(covfunc{:}));

%loghyper0 = -1*ones(nump,1);
%loghyper0 = [zeros(nump-1,1); log(sqrt(0.1))];
loghyper0 = [zeros(nump,1)];

for i=1:abs(numtrials)
  if (i==1) && (numtrials > 0)
    loghyper0 = [zeros(nump,1)];
  else
    loghyper0 = randn(nump,1);
  end
  
  fprintf('*** trial %d/%d ***\n', i, numtrials);
  fprintf('exp(loghyper0) = ');
  fprintf('%g ', exp(loghyper0));
  fprintf('\n');
  
  try
    [loghyper, fX] = minimize(loghyper0, 'gpr', -MAXIT, covfunc, X', Y(:));
  catch
    fprintf('*** ERROR: %s\n', lasterr);

    if (abs(numtrials) == 1)
      error('can not continue!');
    end
    fprintf('continuing anyway...\n');
    loghyper = [];
    fX       = inf;
  end
  mymarg = -fX(end);
  
  fprintf('exp(loghyper)=')
  fprintf('%g ', exp(loghyper));
  fprintf('\n');
  fprintf('marglikelihood = %g\n', mymarg);
  
  all_loghyper0{i} = loghyper0;
  all_marglike(i)  = mymarg;
  all_loghyper{i}  = loghyper;
end

% pick the best
[best, besti] = max(all_marglike);
loghyper  = all_loghyper{besti};
loghyper0 = all_loghyper0{besti};

fprintf('*** best %d: ML=%g\n', besti, best);

gp.loghyper  = loghyper;
gp.covfunc   = covfunc;
gp.X         = X;
gp.Y         = Y;
gp.normmode  = normmode;
gp.loghyper0 = loghyper0;
gp.mll       = best;

if (isempty(loghyper))
  error('no good runs!');
end
