% rvm_demo - demo of the RVM regression functions
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

clear
close all

% reset the random number generator
randn('state',1);
rand('state', 1);

fprintf('*** Train RVM with fixed width ***\n');

% Generate the sinc data
N       = 100;
noise   = 0.1;
X       = 10*[-1:2/(N-1):1];
y       = sin(abs(X))./abs(X);
t       = y + noise*randn(1,N);

% train the RVM (gaussian kernel, bandwidth=3)
rvm   = rvm_train(X, t, 3, '+gauss');

% predict with the RVM
y_rvm = rvm_predict(X, rvm);

% Plot the data and function
figure
subplot(1,2,1)
h_y = plot(X,y,'r--','LineWidth',3);
hold on
h_X = plot(X,t,'k.','MarkerSize',16)
axis([-10.1 10.1 1.1*[min(t) max(t)]]);

% plot the results
h_yrvm = plot(X,y_rvm,'k-','LineWidth',3);
h_rv   = plot(rvm.Xused,t(rvm.used),'ko','LineWidth',2,'MarkerSize',10);
legend([h_X h_y h_yrvm h_rv],'data points', 'sinc function','RVM predictor','RVs')
hold off
grid
title(sprintf('RVM Regression with noisy ''sinc'' data (width=%g)', 3));


% === Train with cross-validation =================================

fprintf('*** Train with Cross-Validation ***\n');

% setup cross validation parameters
widths = {0.5 1 1.5 2 3 4 5 6};
numcv  = 4;
cvsize = 0.9;

% train RVM with cross-validation
rvmcv = rvm_traincv(X, t, widths, '+gauss', '', numcv, cvsize);

% predict with RVM
y_rvmcv = rvm_predict(X, rvmcv);

% plot the data and function
subplot(1,2,2)
h_ycv = plot(X,y,'r--','LineWidth',3);
hold on
h_Xcv = plot(X,t,'k.','MarkerSize',16)
axis([-10.1 10.1 1.1*[min(t) max(t)]]);

% plot the results
h_yrvmcv  = plot(X,y_rvmcv,'k-','LineWidth',3);
h_rvcv    = plot(rvmcv.Xused,t(rvmcv.used),'ko','LineWidth',2,'MarkerSize',10);
legend([h_Xcv h_ycv h_yrvmcv h_rvcv],'data points', 'sinc function','RVM predictor','RVs')
hold off
grid
title(sprintf('RVM Regression with noisy ''sinc'' data using CV (width=%g)', rvmcv.width));


% Output some info
fprintf('\n--- RVM w/ fixed width = 3 ---');
fprintf('\nRVM regression test error (RMS): %g\n', ...
        sqrt(mean((y-y_rvm).^2)))
fprintf('\testimated noise level: %.4f (true: %.4f)\n', ...
        sqrt(1/rvm.beta), noise)

fprintf('\n--- RVM using cross-validation (width=%g) ---', rvmcv.width);
fprintf('\nRVM regression test error (RMS): %g\n', ...
        sqrt(mean((y-y_rvmcv).^2)))
fprintf('\testimated noise level: %.4f (true: %.4f)\n', ...
        sqrt(1/rvmcv.beta), noise)


