function rvm = rvm_train(X, Y, width, kernel_, normmode)
% rvm_train - train an RVM regression function
%
%    rvm = rvm_train(X, Y, width, kernel, normmode)
%
%         X = feature vectors (each column is a feature vector)
%         Y = function value
%     width = the kernel width
%    kernel = the kernel function (default = '+gauss')
%  normmode = normalization [default = '']
%             'y' - make Y zero-mean
%
%       rvm = the RVM
%
%  Train a regression function Y = f(X) using RVM package.
%  For more kernels see "sbl_kernelFunction".
%
% This uses the RVM code by Mike Tipping.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD


if (nargin<4)
  kernel_ = '+gauss';
end
if (nargin<5)
  normmode = '';
end

setEnvironment('InfoLevel',3);

% normalize data
if any(normmode == 'y')
  Ymean = mean(Y);
  Y = Y - Ymean;
else
  Ymean = [];
end

% RVM parameters
maxIts        = 1000;
monIts        = round(maxIts/10);

% Set up initial hyperparameters -
% should be relatively insensitive to exact values
% 
N               = size(X,2);
initAlpha       = (1/N)^2;
epsilon         = std(Y) * 10/100;
if (epsilon < 1e-6)
  warning('epsilon is small, regularizing');
  epsilon  = 1e-6;
  keyboard
end
initBeta        = 1/epsilon^2;

%
% Train a relevance vector machine
% 
[weights, used, ml, alpha, beta] = ...
    sbl_rvm(X',Y(:),initAlpha,initBeta,kernel_,width,maxIts,monIts);

rvm.kernel_orig = kernel_;

%
% Visualise the results
% 
if kernel_(1)=='+'
  if (length(used)>0)
    % Take account of bias if originally used ...
    used  = used - 1;
    if used(1)~=0
      % ... and if pruned ...
      kernel_(1)  = [];
    else
      used(1)     = [];
    end
  end
end

rvm.kernel  = kernel_;
rvm.width   = width;
rvm.Xused   = X(:,used);
rvm.weights = weights;
rvm.used    = used;
rvm.alpha   = alpha;
rvm.beta    = beta;
rvm.ml      = ml;
rvm.Ymean   = Ymean;


