function rvm = rvm_traincv(X, Y, widths, kernel_, normmode, numcv, cvsize)
% rvm_traincv - train an RVM regression function w/ cross-validation
%
%    rvm = rvm_traincv(X, Y, widths, kernel, normmode, numcv, cvsize)
%    rvm = rvm_traincv(X, Y, widths, kernel, normmode, cvset)
%
%      INPUT
%        X  = feature vectors (each column is a feature vector)
%        Y  = function value
%   widths  = cell array of kernel widths for cross validation
%   kernel  = the kernel function (default = '+gauss')
%  normmode = normalization [default = '']
%             'y' - make Y zero-mean
%
%  Cross validation is specified in two ways:
%  1) the number of splits and the size used for training
%    numcv  = number of cross-validations (e.g. 4)
%    cvsize = amount of data used for cross-validation training (e.g. 0.8)
%  2) a pre-specified list of indexes to the training data
%     cvset = cell array: {{train1, cv1}, {train2, cv2}, ...}
%
%    OUTPUT
%      rvm = the RVM
%
%  See "rvm_train" for more info.
%  For more kernels see "sbl_kernelFunction".
%
% This uses the RVM code by Mike Tipping.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

if (iscell(numcv))
  cvset  = numcv;
  clear numcv;
  numcv  = length(cvset);
  cvmode = 1;
else
  cvmode = 0;
end

N      = length(Y);
numw   = length(widths);
mseall = zeros(numw, numcv);
mlall  = zeros(numw, numcv);

%%% loop through cross-validations %%%
for cv=1:numcv
  
  if (cvmode == 0)
    % build cv set
    tmp = randperm(N);
    trainset = tmp(1:round(cvsize*N));
    testset  = tmp((round(cvsize*N)+1):N);
  elseif (cvmode == 1)
    trainset = cvset{cv}{1};
    testset  = cvset{cv}{2};
  else
    error('bad cvmode');
  end
    
    
  %%% loop through each width %%%%%
  for wi = 1:length(widths)
    fprintf('--- cv=%d wi=%d -------\n', cv, wi);
    mywidth = widths{wi};

    try
      % train an RVM
      myrvm  = rvm_train(X(:,trainset), Y(trainset), mywidth, ...
			 kernel_, normmode);
      
      % check for invalid RVM
      if (length(myrvm.used) == 0)
	mseall(wi, cv) = inf;
	fprintf('* Invalid RVM\n');
      
      else
	% test RVM on testset
	myyhat = rvm_predict(X(:,testset), myrvm);
  
	% compute MSE
	mseall(wi, cv) = mean( (myyhat-Y(testset)).^2 );
	mlall(wi, cv)  = myrvm.ml;
      end
	
    catch
      fprintf('*** ERROR: %s\n', lasterr);
      mseall(wi, cv) = inf;
    end
      
  end
end

% compute average MSE
mmseall = mean(mseall,2);

%%% output MSE table %%%%%%%%%%%%%%%%%%%%
fprintf('\n* Cross-validation results\n');
for i=1:length(widths)
  fprintf('width = [');
  fprintf('%g ', widths{i});
  fprintf('] : ');
  fprintf('%0.5g  (', mmseall(i));
  fprintf('%0.5g ', mseall(i,:));
  fprintf(')\n');
end


%%% pick best %%%%%%%%%%%%%%%%%%%%%%%%%%%

% sort by mean MSE
[tmp, inds] = sort(mmseall);
inds = inds(:)';

% try each width in order
for besti=inds
  best = mmseall(besti);
  
  % check for error
  if isinf(best)
    error('bad cross-validation!');
  end

  fprintf('--- final training ---------\n');
  fprintf('best = %d (%g)\n', besti, best);
  fprintf('width = ');
  fprintf('%g ', widths{besti});
  fprintf('\n');


  %%% retrain with all data %%%%%%%%%%%%%%%
  rvm = rvm_train(X, Y, widths{besti}, kernel_, normmode);

  % check for valid RVM
  if (length(rvm.used) > 0)
    break;
    
  else
    fprintf('* invalid RVM, trying next width\n');
    rvm = [];
  end
end


% check for no RVM
if isempty(rvm)
  error('could not train RVM!');
end

rvm.param.width  = widths{besti};
rvm.param.widths = widths;
rvm.mseall       = mseall;
