function mysvr = svrl_traincv(X, Y, Cs, es, kernel, widths, numcv, cvsize)
%   svrl_traincv - train an SVR regression function with cross-validation
%
%     svr = svrl_traincv(X, Y, Cs, es, kernel, widths, numcv, cvsize)
% 
%    INPUT
%        X = feature vectors (each column is a feature vector)
%        Y = function value
%       Cs = trade-off parameters (array)
%       es = epsilon insensitivities (array)
%   kernel = the kernel function (default = 2 (rbf))
%   widths = the kernel widths (cell array)
%    numcv = number of CV trials
%   cvsize = amount of data to use for CV training
%
%   OUTPUT
%      svr = SV regression model
%
%  see "svml" for more kernel functions
%
% This function uses the SVMlight library.
%
% Copyright 2008, Antoni Chan, SVCL, UCSD

N      = length(Y);
numw   = length(widths);
nume   = length(es);
numC   = length(Cs)

% form all combos
params = {};
for i=1:nume
  for j=1:numC
    for k=1:numw
      params{end+1} = {es(i), Cs(j), widths{k}};
    end
  end
end
nump = length(params)
params

mseall = zeros(nump, numcv);

%%% loop through cross-validations %%%
for cv=1:numcv
  
  % build cv set
  tmp = randperm(N);
  trainset = tmp(1:round(cvsize*N));
  testset  = tmp((round(cvsize*N)+1):N);
  
  %%% loop through each param set %%%%%
  for p = 1:nump
    fprintf('--- cv=%d p=%d -------\n', cv, p);
    myparam = params{p};

    % train an SVR, then test
    mysvr  = svrl_train(X(:,trainset), Y(trainset), myparam{2}, myparam{1}, ...
			kernel, myparam{3});
    myyhat = svrl_predict(X(:,testset), mysvr);
  
    % compute MSE
    mseall(p, cv) = mean( (myyhat-Y(testset)).^2 );
  end
end

% average over all runs
mmseall = mean(mseall,2);

%%% output MSE table %%%%%%%%%%%%%%%%%%%%
fprintf('\n* Cross-validation results\n');
for i=1:length(params)
  fprintf('eps=%g; C=%g; ', params{i}{1}, params{i}{2});
  fprintf('width = [');
  fprintf('%g ', params{i}{3});
  fprintf('] : ');
  fprintf('%0.5g  (', mmseall(i));
  fprintf('%0.5g ', mseall(i,:));
  fprintf(')\n');
end



%%% pick best %%%%%%%%%%%%%%%%%%%%%%%%%%%
[best, besti] = min(mmseall);

fprintf('--- final training ---------\n');
fprintf('best = %d (%g)\n', besti, best);
fprintf('C = %g; e=%g; ', params{besti}{2}, params{besti}{1});
fprintf('width = ');
fprintf('%g ', params{besti}{3});
fprintf('\n');


%%% retrain with all data %%%%%%%%%%%%%%%
mysvr  = svrl_train(X(:,trainset), Y(trainset), params{besti}{2}, params{besti}{1}, ...
		    kernel, params{besti}{3});

mysvr.param.C      = params{besti}{2};
mysvr.param.e      = params{besti}{1};
mysvr.param.width  = params{besti}{3};
mysvr.param.Cs     = Cs;
mysvr.param.es     = es;
mysvr.param.widths = widths;
mysvr.mseall       = mseall;
